# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

from twisted.trial.unittest import TestCase

import pkg_resources
import os

from collections import OrderedDict

from .._builder import render_fragments, split_fragments
from .._writer import append_to_newsfile


class WritingTests(TestCase):

    def test_append_at_top(self):

        fragments = {
            "": {
                "142.misc": u"",
                "1.misc": u"",
                "4.feature": u"Stuff!",
                "2.feature": u"Foo added.",
                "72.feature": u"Foo added.",
            },
            "Web": {
                "3.bugfix": u"Web fixed.",
            },
            "Names": {}
        }

        definitions = OrderedDict([
            ("feature", {"name": "Features", "showcontent": True}),
            ("bugfix", {"name": "Bugfixes", "showcontent": True}),
            ("misc", {"name": "Misc", "showcontent": False}),
        ])

        expected_output = """MyProject 1.0
=============

Features
--------

- Foo added. (#2, #72)
- Stuff! (#4)


Misc
----

- #1, #142


Names
-----

No significant changes.


Web
---

Bugfixes
~~~~~~~~

- Web fixed. (#3)


Old text.
"""

        tempdir = self.mktemp()
        os.mkdir(tempdir)

        with open(os.path.join(tempdir, "NEWS.rst"), "w") as f:
            f.write("Old text.\n")

        fragments = split_fragments(fragments, definitions)

        template = pkg_resources.resource_string(
            "towncrier",
            "templates/template.rst").decode('utf8')

        append_to_newsfile(tempdir,
                           "NEWS.rst",
                           ".. towncrier release notes start\n",
                           "MyProject 1.0\n=============\n",
                           render_fragments(template, fragments, definitions))

        with open(os.path.join(tempdir, "NEWS.rst"), "r") as f:
            output = f.read()

        self.assertEqual(expected_output, output)

    def test_append_at_top_with_hint(self):
        """
        If there is a comment with C{.. towncrier release notes start},
        towncrier will add the version notes after it.
        """
        fragments = {
            "": {
                "142.misc": u"",
                "1.misc": u"",
                "4.feature": u"Stuff!",
                "2.feature": u"Foo added.",
                "72.feature": u"Foo added.",
                "99.feature": u"Foo! " * 100
            },
            "Web": {
                "3.bugfix": u"Web fixed.",
            },
            "Names": {}
        }

        definitions = OrderedDict([
            ("feature", {"name": "Features", "showcontent": True}),
            ("bugfix", {"name": "Bugfixes", "showcontent": True}),
            ("misc", {"name": "Misc", "showcontent": False}),
        ])

        expected_output = """Hello there! Here is some info.

.. towncrier release notes start

MyProject 1.0
=============

Features
--------

- Foo added. (#2, #72)
- Stuff! (#4)
- Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo!
  Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo!
  Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo!
  Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo!
  Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo!
  Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo!
  Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! Foo! (#99)


Misc
----

- #1, #142


Names
-----

No significant changes.


Web
---

Bugfixes
~~~~~~~~

- Web fixed. (#3)


Old text.
"""

        tempdir = self.mktemp()
        os.mkdir(tempdir)

        with open(os.path.join(tempdir, "NEWS.rst"), "w") as f:
            f.write(("Hello there! Here is some info.\n\n"
                     ".. towncrier release notes start\nOld text.\n"))

        fragments = split_fragments(fragments, definitions)

        template = pkg_resources.resource_string(
            "towncrier",
            "templates/template.rst").decode('utf8')

        append_to_newsfile(tempdir,
                           "NEWS.rst",
                           ".. towncrier release notes start\n",
                           "MyProject 1.0\n=============\n",
                           render_fragments(template, fragments, definitions))

        with open(os.path.join(tempdir, "NEWS.rst"), "r") as f:
            output = f.read()

        self.assertEqual(expected_output, output)



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

"""
Responsible for getting the version and name from a project.
"""

from __future__ import absolute_import, division

import sys

from importlib import import_module

from incremental import Version


def _get_package(package_dir, package):

    # Step 1: Try the dumbest and simplest thing that could possibly work.
    # Yes, that means importing it. Call the cops, I don't care.

    sys.path.insert(1, package_dir)

    try:
        module = import_module(package)
    except ImportError as e:
        print("Tried to import {}, but ran into this error: {}".format(package,
                                                                       e))
        # wups that didn't work
        module = None

    # Don't leave trash in sys.path
    sys.path.pop(0)

    # Step 2: uhhhhhhh
    # TBA

    if not module:
        raise Exception("Can't find your project :(")

    return module


def get_version(package_dir, package):

    module = _get_package(package_dir, package)

    version = getattr(module, "__version__", None)

    if not version:
        raise Exception("No __version__, I don't know how else to look")

    if isinstance(version, str):
        return version

    if isinstance(version, Version):
        return version.base()

    if isinstance(version, tuple):
        return ".".join(map(str, version))

    raise Exception(("I only know how to look at a __version__ that is a str, "
                     "an Increment Version, or a tuple. If you can't provide "
                     "that, use the --version argument and specify one."))


def get_project_name(package_dir, package):

    module = _get_package(package_dir, package)

    version = getattr(module, "__version__", None)

    if not version:
        # welp idk
        return package.title()

    if isinstance(version, str):
        return package.title()

    if isinstance(version, Version):
        # Incremental has support for package names
        return version.package



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

import os
from twisted.trial.unittest import TestCase

from click.testing import CliRunner
from .. import _main


class TestCli(TestCase):

    maxDiff = None

    def test_happy_path_ini(self):
        runner = CliRunner()

        with runner.isolated_filesystem():
            with open('towncrier.ini', 'w') as f:
                f.write(
                    '[towncrier]\n'
                    'package = foo\n'
                    'package_dir = .\n'
                )
            os.mkdir('foo')
            with open('foo/__init__.py', 'w') as f:
                f.write('__version__ = "1.2.3"\n')
            os.mkdir('foo/newsfragments')
            with open('foo/newsfragments/123.feature', 'w') as f:
                f.write('Adds levitation')

            result = runner.invoke(_main, ['--draft', '--date', '01-01-2001'])

        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            result.output,
            u'Loading template...\nFinding news fragments...\nRendering news '
            u'fragments...\nDraft only -- nothing has been written.\nWhat is '
            u'seen below is what would be written.\n\nFoo 1.2.3 (01-01-2001)'
            u'\n======================\n'
            u'\n\nFeatures\n--------\n\n- Adds levitation (#123)\n\n'
        )

    def test_happy_path_toml(self):
        runner = CliRunner()

        with runner.isolated_filesystem():
            with open('pyproject.toml', 'w') as f:
                f.write(
                    '[tool.towncrier]\n'
                    'package = "foo"\n'
                )
            os.mkdir('foo')
            with open('foo/__init__.py', 'w') as f:
                f.write('__version__ = "1.2.3"\n')
            os.mkdir('foo/newsfragments')
            with open('foo/newsfragments/123.feature', 'w') as f:
                f.write('Adds levitation')

            result = runner.invoke(_main, ['--draft', '--date', '01-01-2001'])

        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            result.output,
            u'Loading template...\nFinding news fragments...\nRendering news '
            u'fragments...\nDraft only -- nothing has been written.\nWhat is '
            u'seen below is what would be written.\n\nFoo 1.2.3 (01-01-2001)'
            u'\n======================\n'
            u'\n\nFeatures\n--------\n\n- Adds levitation (#123)\n\n'
        )



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

import os

from twisted.trial.unittest import TestCase

from .._project import get_version


class VersionFetchingTests(TestCase):

    def test_str(self):
        """
        A str __version__ will be picked up.
        """
        temp = self.mktemp()
        os.makedirs(temp)
        os.makedirs(os.path.join(temp, "mytestproj"))

        with open(os.path.join(temp, "mytestproj", "__init__.py"), "w") as f:
            f.write("__version__ = '1.2.3'")

        version = get_version(temp, "mytestproj")
        self.assertEqual(version, "1.2.3")

    def test_tuple(self):
        """
        A tuple __version__ will be picked up.
        """
        temp = self.mktemp()
        os.makedirs(temp)
        os.makedirs(os.path.join(temp, "mytestproja"))

        with open(os.path.join(temp, "mytestproja", "__init__.py"), "w") as f:
            f.write("__version__ = (1, 3, 12)")

        version = get_version(temp, "mytestproja")
        self.assertEqual(version, "1.3.12")



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

from __future__ import absolute_import, division

import pkg_resources

from twisted.trial.unittest import TestCase

from collections import OrderedDict

from .._builder import render_fragments, split_fragments, normalise


class FormatterTests(TestCase):

    def test_normalise(self):

        cases = [
            ("   hello", "hello"),
            ("\thello\nthere\n people", "hello there people"),
            ("hi\nthere        what's up\n\n\n", "hi there what's up"),
        ]

        for case in cases:
            self.assertEqual(normalise(case[0]), case[1])

    def test_split(self):

        fragments = {
            "": {
                "1.misc": u"",
                "baz.misc": u"",
                "2.feature": u"Foo added.",
                "421.feature~": u"Foo added.",
                "5.feature": u"Foo added.    \n",
                "6.bugfix": u"Foo added.",
                "NEWS": u"Some junk.",
            },
            "Web": {
                "3.bugfix": u"Web fixed.    ",
                "4.feature": u"Foo added."
            }
        }

        expected_output = {
            "": {
                "misc": {
                    "": ["1", "baz"],
                },
                "feature": {
                    u"Foo added.": ["2", "5"]
                },
                "bugfix": {
                    u"Foo added.": ["6"]
                }
            },
            "Web": {
                "bugfix": {
                    u"Web fixed.": ["3"],
                },
                "feature": {
                    u"Foo added.": ["4"]
                }
            }

        }

        definitions = OrderedDict([
            ("feature", {"name": "Features", "showcontent": True}),
            ("bugfix", {"name": "Bugfixes", "showcontent": True}),
            ("misc", {"name": "Misc", "showcontent": False}),
        ])

        output = split_fragments(fragments, definitions)

        self.assertEqual(expected_output, output)

    def test_basic(self):
        """
        Basic functionality -- getting a bunch of news fragments and formatting
        them into a rST file -- works.
        """
        fragments = {
            "": {
                "142.misc": u"",
                "1.misc": u"",
                "bar.misc": u"",
                "4.feature": u"Stuff!",
                "2.feature": u"Foo added.",
                "72.feature": u"Foo added.",
                "baz.feature": u"Fun!",
            },
            "Web": {
                "3.bugfix": u"Web fixed.",
            },
            "Names": {}
        }

        definitions = OrderedDict([
            ("feature", {"name": "Features", "showcontent": True}),
            ("bugfix", {"name": "Bugfixes", "showcontent": True}),
            ("misc", {"name": "Misc", "showcontent": False}),
        ])

        expected_output = (u"""
Features
--------

- Foo added. (#2, #72)
- Stuff! (#4)
- Fun! (baz)


Misc
----

- #1, #142, bar


Names
-----

No significant changes.


Web
---

Bugfixes
~~~~~~~~

- Web fixed. (#3)
""")

        template = pkg_resources.resource_string(
            "towncrier",
            "templates/template.rst").decode('utf8')

        fragments = split_fragments(fragments, definitions)
        output = render_fragments(template, fragments, definitions)
        self.assertEqual(output, expected_output)



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

from subprocess import call

import os
import click


def remove_files(base_dir, fragment_directory, sections, fragments):
    to_remove = []

    for section_name, categories in fragments.items():

        if fragment_directory is not None:
            section_dir = os.path.join(base_dir, sections[section_name],
                                       fragment_directory)
        else:
            section_dir = os.path.join(base_dir, sections[section_name])

        for category_name, category_items in categories.items():

            for tickets in category_items.values():

                for ticket in tickets:

                    filename = str(ticket) + "." + category_name
                    to_remove.append(os.path.join(
                        section_dir, filename))

    if not to_remove:
        return

    click.echo("I want to remove the following files:")

    for filename in to_remove:
        click.echo(filename)

    if click.confirm('Is it okay if I remove those files?', default=True):
        call(["git", "rm", "--quiet"] + to_remove)


def stage_newsfile(directory, filename):

    call(["git", "add", os.path.join(directory, filename)])



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

from twisted.trial.unittest import TestCase

import os

from .._settings import load_config


class IniSettingsTests(TestCase):

    def test_base(self):
        """
        Test a "base config" -- with just a package name.
        """
        temp = self.mktemp()
        os.makedirs(temp)

        with open(os.path.join(temp, "towncrier.ini"), "w") as f:
            f.write("""[towncrier]
package = foobar
""")

        config = load_config(temp)
        self.assertEqual(config['package'], "foobar")
        self.assertEqual(config['package_dir'], ".")
        self.assertEqual(config['filename'], "NEWS.rst")


class TomlSettingsTests(TestCase):

    def test_base(self):
        """
        Test a "base config".
        """
        temp = self.mktemp()
        os.makedirs(temp)

        with open(os.path.join(temp, "pyproject.toml"), "w") as f:
            f.write("""[tool.towncrier]
package = "foobar"
""")

        config = load_config(temp)
        self.assertEqual(config['package'], "foobar")
        self.assertEqual(config['package_dir'], ".")
        self.assertEqual(config['filename'], "NEWS.rst")



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

"""
towncrier, a builder for your news files.
"""

from __future__ import absolute_import, division

import os
import click
import pkg_resources

from datetime import date

from ._settings import load_config
from ._builder import find_fragments, split_fragments, render_fragments
from ._project import get_version, get_project_name
from ._writer import append_to_newsfile
from ._git import remove_files, stage_newsfile
from ._version import __version__


def _get_date():
    return date.today().isoformat()


@click.command()
@click.option('--draft', 'draft', default=False, flag_value=True,
              help=("Render the news fragments, don't write to files, "
                    "don't check versions."))
@click.option('--dir', 'directory', default='.')
@click.option('--version', 'project_version', default=None)
@click.option('--date', 'project_date', default=None)
def _main(draft, directory, project_version, project_date):
    return __main(draft, directory, project_version, project_date)


def __main(draft, directory, project_version, project_date):
    """
    The main entry point.
    """
    directory = os.path.abspath(directory)
    config = load_config(directory)
    to_err = draft

    click.echo("Loading template...", err=to_err)
    if config['template'] is None:
        template = pkg_resources.resource_string(
            __name__,
            "templates/template.rst").decode('utf8')
    else:
        with open(config['template'], 'rb') as tmpl:
            template = tmpl.read().decode('utf8')

    click.echo("Finding news fragments...", err=to_err)

    definitions = config['types']

    if config.get("directory"):
        base_directory = os.path.abspath(config["directory"])
        fragment_directory = None
    else:
        base_directory = os.path.abspath(os.path.join(
            directory, config['package_dir'], config['package']))
        fragment_directory = "newsfragments"

    fragments = find_fragments(
        base_directory, config['sections'], fragment_directory)

    click.echo("Rendering news fragments...", err=to_err)

    fragments = split_fragments(fragments, definitions)
    rendered = render_fragments(template, fragments, definitions)

    if not project_version:
        project_version = get_version(
            os.path.abspath(os.path.join(directory, config['package_dir'])),
            config['package'])

    project_name = get_project_name(
        os.path.abspath(os.path.join(directory, config['package_dir'])),
        config['package'])

    if project_date is None:
        project_date = _get_date()

    top_line = config['title_format'].format(
        name=project_name,
        version=project_version,
        project_date=project_date
    )
    top_line += u"\n" + (u"=" * len(top_line)) + u"\n"

    if draft:
        click.echo(
            "Draft only -- nothing has been written.\n"
            "What is seen below is what would be written.\n",
            err=to_err,
        )
        click.echo(top_line, err=to_err)
        click.echo(rendered)
    else:
        click.echo("Writing to newsfile...", err=to_err)
        start_line = config['start_line']
        append_to_newsfile(directory, config['filename'],
                           start_line, top_line, rendered)

        click.echo("Staging newsfile...", err=to_err)
        stage_newsfile(directory, config['filename'])

        click.echo("Removing news fragments...", err=to_err)
        remove_files(
            base_directory, fragment_directory, config['sections'], fragments)

        click.echo("Done!", err=to_err)


__all__ = ["__version__"]



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

import os
import toml

from collections import OrderedDict

try:
    import configparser
except ImportError:
    import ConfigParser as configparser


_start_string = u'.. towncrier release notes start\n'
_title_format = u'{name} {version} ({project_date})'
_template_fname = None
_default_types = OrderedDict([
    (u"feature", {"name": u"Features", "showcontent": True}),
    (u"bugfix", {"name": u"Bugfixes", "showcontent": True}),
    (u"doc", {"name": u"Improved Documentation", "showcontent": True}),
    (u"removal", {"name": u"Deprecations and Removals",
                  "showcontent": True}),
    (u"misc", {"name": u"Misc", "showcontent": False}),
])


def load_config_ini(from_dir):

    config = configparser.ConfigParser(
        {
            'package_dir': '.',
            'filename': 'NEWS.rst',
            'directory': None,
        }
    )
    config.read(os.path.join(from_dir, "towncrier.ini"))

    if 'towncrier' not in config.sections():
        raise ValueError("No [towncrier] section.")

    if 'package' not in config.options('towncrier'):
        raise ValueError(
            "The [towncrier] section has no required 'package' key.")

    try:
        start_string = config.get('towncrier', 'start_string')
    except configparser.NoOptionError:
        start_string = _start_string

    try:
        title_format = config.get('towncrier', 'title_format')
    except configparser.NoOptionError:
        title_format = _title_format

    try:
        template_fname = config.get('towncrier', 'template')
    except configparser.NoOptionError:
        template_fname = None

    return {
        'package': config.get('towncrier', 'package'),
        'package_dir': config.get('towncrier', 'package_dir'),
        'filename': config.get('towncrier', 'filename'),
        'directory': config.get('towncrier', 'directory'),
        'sections': {'': ''},
        'types': _default_types,
        'template': template_fname,
        'start_line': start_string,
        'title_format': title_format,
    }


def load_config_toml(from_dir):
    fn = os.path.join(from_dir, "pyproject.toml")
    if not os.path.exists(fn):
        return None
    with open(fn, 'r') as conffile:
        config = toml.load(conffile)

    if 'tool' not in config:
        raise ValueError("No [tool.towncrier] section.")

    config = config['tool']['towncrier']

    if 'package' not in config:
        raise ValueError(
            "The [tool.towncrier] section has no required 'package' key.")

    sections = OrderedDict()
    types = OrderedDict()

    if "section" in config:
        for x in config["section"]:
            sections[x.get('name', '')] = x['path']
    else:
        sections[''] = ''

    if "type" in config:
        for x in config["type"]:
            types[x["directory"]] = {"name": x["name"],
                                     "showcontent": x["showcontent"]}
    else:
        types = _default_types

    return {
        'package': config.get('package'),
        'package_dir': config.get('package_dir', '.'),
        'filename': config.get('filename', 'NEWS.rst'),
        'directory': config.get('directory'),
        'sections': sections,
        'types': types,
        'template': config.get('template', _template_fname),
        'start_line': config.get('start_string', _start_string),
        'title_format': config.get('title_format', _title_format),
    }


def load_config(from_dir):
    res = load_config_toml(from_dir)
    if res is None:
        return load_config_ini(from_dir)
    return res



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

from __future__ import absolute_import, division, print_function

import os
import textwrap

from collections import OrderedDict

from jinja2 import Template


def normalise(text):

    # Blitz newlines
    text = text.replace(u"\r\n", u"\n")
    text = text.replace(u"\n", u" ")

    # No tabs!
    text = text.replace(u"\t", u" ")

    # Remove double spaces
    while u"  " in text:
        text = text.replace(u"  ", u" ")

    # Remove left/right whitespace
    text = text.strip()

    return text


def find_fragments(base_directory, sections, fragment_directory):
    """
    Sections are a dictonary of section names to paths.
    """
    content = {}

    for key, val in sections.items():

        if fragment_directory is not None:
            section_dir = os.path.join(base_directory, val, fragment_directory)
        else:
            section_dir = os.path.join(base_directory, val)

        files = os.listdir(section_dir)

        file_content = {}

        for fragment in files:
            with open(os.path.join(section_dir, fragment), "rb") as f:
                file_content[fragment] = f.read().decode('utf8', 'replace')

        content[key] = file_content

    return content


def split_fragments(fragments, definitions):

    output = {}

    for section_name, section_fragments in fragments.items():
        section = {}

        for filename, content in section_fragments.items():

            content = normalise(content)
            parts = filename.split(u".")

            if len(parts) == 1:
                continue
            else:
                ticket, category = parts

            if category not in definitions:
                continue

            if definitions[category]["showcontent"] is False:
                content = u""

            texts = section.get(category, {})

            if texts.get(content):
                texts[content] = sorted(texts[content] + [ticket])
            else:
                texts[content] = [ticket]

            section[category] = texts

        output[section_name] = section

    return output


def render_fragments(template, fragments, definitions, major=u"-", minor=u"~"):
    """
    Render the fragments into a news file.
    """

    jinja_template = Template(template, trim_blocks=True)

    data = {}

    for section_name, section_value in fragments.items():

        data[section_name] = {}

        for category_name, category_value in section_value.items():
            categories = OrderedDict()

            for text, tickets in category_value.items():
                ticket_numbers = []

                for ticket in tickets:
                    try:
                        int(ticket)
                        ticket_numbers.append(u"#" + ticket)
                    except:
                        ticket_numbers.append(ticket)

                categories[text] = ticket_numbers

            data[section_name][category_name] = categories

    done = []

    res = jinja_template.render(sections=data, definitions=definitions)

    for line in res.split(u"\n"):
        done.append(textwrap.fill(line, width=79, subsequent_indent=u"  "))

    return u"\n".join(done).rstrip() + u"\n"



# Copyright (c) Amber Brown, 2015
# See LICENSE for details.

"""
Responsible for writing the built news fragments to a news file without
affecting existing content.
"""

from __future__ import absolute_import, division

import os


def append_to_newsfile(directory, filename, start_line, top_line, content):

    news_file = os.path.join(directory, filename)

    if not os.path.exists(news_file):
        existing_content = u""
    else:
        with open(news_file, "rb") as f:
            existing_content = f.read().decode('utf8')

    existing_content = existing_content.split(start_line, 1)

    if top_line in existing_content:
        raise ValueError(
            "It seems you've already produced newsfiles for this version?")

    with open(os.path.join(directory, filename), "wb") as f:

        if len(existing_content) > 1:
            f.write(existing_content.pop(0).rstrip().encode('utf8'))
            f.write((u"\n\n" + start_line + u"\n").encode('utf8'))

        f.write(top_line.encode('utf8'))
        f.write(content.encode('utf8'))
        if existing_content[0]:
            f.write(b"\n\n")
        f.write(existing_content[0].lstrip().encode('utf8'))



#!/usr/bin/env python

from __future__ import absolute_import, division, print_function

import os, sys

from setuptools import setup, find_packages

setup(
    name='towncrier',
    maintainer='Amber Brown',
    maintainer_email='hawkowl@twistedmatrix.com',
    url="https://github.com/hawkowl/towncrier",
    classifiers = [
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
    ],
    use_incremental=True,
    setup_requires=['incremental'],
    install_requires=[
        'Click',
        'incremental',
        'jinja2',
        'toml',
    ],
    package_dir={"": "src"},
    packages=find_packages('src'),
    license="MIT",
    zip_safe=False,
    include_package_data=True,
    description='Building newsfiles for your project.',
    long_description=open('README.rst').read(),
    entry_points={
        'console_scripts': [
            'towncrier = towncrier:_main',
        ],
    }
)
